#include "PerspectiveView.h"
#include <QSplitter>
#include <QVBoxLayout>
#include <QDebug>
#include <QGroupBox>
#include <internal_impl.h>
#include <componentinfo_access.h>

void PerspectiveView::deduce_start()
{
	auto request = std::make_shared<DeduceRequest>();
	request->reset(-1, RtEntityManager::const_id);
	emit this->backend_deduce_request(request);
}

void PerspectiveView::reply_accept(const QList<std::shared_ptr<WsMessage>>& msg_set)
{
	for (auto msg : msg_set)
	{
		if (msg->topicString() == TypesQueryResult().topicString()) {
			_templet_model->clear();

			auto ins_conv = std::dynamic_pointer_cast<TypesQueryResult>(msg);
			for (auto sign : ins_conv->_entity_templets.keys()) {
				auto cell = new QStandardItem(sign);
				cell->setData(ins_conv->_entity_templets[sign]);
				cell->setEditable(false);
				_templet_model->appendRow(cell);
			}
		}
		if (msg->topicString() == ProcedureSignatureQueryResult().topicString()) {
			this->_procedure_model->removeRows(0, this->_procedure_model->rowCount());

			auto ins_conv = std::dynamic_pointer_cast<ProcedureSignatureQueryResult>(msg);
			for (auto fun_pair : ins_conv->_signature_list) {
				QList<QStandardItem*> row;
				row << new QStandardItem(fun_pair.first);
				row << new QStandardItem(fun_pair.second);
				for(auto info : row) info->setEditable(false);
				
				this->_procedure_model->appendRow(row);
			}
			this->_procedure_view->resizeColumnsToContents();
		}
	}
}

void PerspectiveView::complete_accept(std::shared_ptr<RespondDefault> ins) {}

void PerspectiveView::show_procedures_start(const QModelIndex& target)
{
	if (!target.isValid())
		return;

	auto templet_id = this->_templet_model->itemFromIndex(target)->data().toULongLong();
	auto ins = std::make_shared<ProcedureSignatureQuery>();
	ins->reset(-1, templet_id);
	_bind_framework->accept(QList<std::shared_ptr<WsMessage>>{ins});
}

PerspectiveView::PerspectiveView(std::shared_ptr<DeduceFramework> ins, QWidget* p /*= nullptr*/)
	:QDialog(p),
	_bind_framework(ins),
	_bind_timer(new QTimer(this)),
	_templet_types(new QListView(this)),
	_templet_model(new QStandardItemModel(this)),
	_procedure_view(new QTableView(this)),
	_procedure_model(new QStandardItemModel(this))
{
	connect(this, &PerspectiveView::backend_deduce_request, _bind_framework.get(), &DeduceFramework::deduceBegin);
	connect(_bind_framework.get(), &DeduceFramework::reply, this, &PerspectiveView::reply_accept);
	connect(_bind_framework.get(), &DeduceFramework::complete, this, &PerspectiveView::complete_accept);
	connect(_templet_types, &QListView::clicked, this, &PerspectiveView::show_procedures_start);

	this->setWindowTitle(u8"͸ͼ");
	this->setMinimumSize(800, 600);

	auto layout = new QVBoxLayout(this);
	auto splitter = new QSplitter(this);
	layout->addWidget(splitter);

	splitter->addWidget(_templet_types);
	auto groupbox = new QGroupBox(u8"ùǩ", this);
	auto glayout = new QVBoxLayout(groupbox);
	glayout->addWidget(_procedure_view);
	_procedure_view->setModel(_procedure_model);
	splitter->addWidget(groupbox);
	_templet_types->setModel(_templet_model);

	splitter->setStretchFactor(1, 1);

	auto templet_pull = std::make_shared<TypesQuery>();
	templet_pull->reset(-1, RtEntityManager::const_id);
	_bind_framework->accept(QList<std::shared_ptr<WsMessage>>() << templet_pull);
	connect(_bind_timer, &QTimer::timeout, this, &PerspectiveView::deduce_start);

	_bind_timer->start(100);
	_procedure_model->setHorizontalHeaderLabels(QStringList()<<u8"Ϣ"<<u8"Ϣ");
}
